<?php

declare(strict_types=1);

namespace Evidente\Aircash\Console\Command;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Factory\Partner\CertificateFactoryInterface;
use Magento\Framework\App\Area;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\App\State;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Store\Model\ScopeInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Exception;
use Webmozart\Assert\Assert;

/**
 * @SuppressWarnings(Long)
 * @SuppressWarnings(Coupling)
 * @SuppressWarnings(Static)
 * @codeCoverageIgnore
 */
class ActivateCommand extends Command
{
    /**
     * @var State
     */
    private $state;

    /**
     * @var CertificateFactoryInterface
     */
    private $certificateFactory;

    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     * @var WriterInterface
     */
    private $writer;

    /**
     * @var TypeListInterface
     */
    private $cacheTypeList;

    /**
     * @var EncryptorInterface
     */
    private $encryptor;

    public function __construct(
        State $appState,
        CertificateFactoryInterface $certificateFactory,
        ConfigInterface $config,
        WriterInterface $writer,
        TypeListInterface $cacheTypeList,
        EncryptorInterface $encryptor
    ) {
        $this->state = $appState;
        $this->certificateFactory = $certificateFactory;
        $this->config = $config;
        $this->writer = $writer;
        $this->cacheTypeList = $cacheTypeList;
        $this->encryptor = $encryptor;

        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('evidente:aircash:activate')
            ->setDescription('Make activation')
            ->addOption('store-id', null, InputOption::VALUE_REQUIRED);

        parent::configure();
    }

    /**
     * @return int|null
     * @throws Exception
     * @SuppressWarnings(UnusedFormalParameter)
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ) {
        try {
            $this->state->setAreaCode(Area::AREA_ADMINHTML);
        } catch (Exception $exception) {
            return 0;
        }

        $this->cleanCache();

        $storeId = $input->getOption('store-id');

        Assert::nullOrNumeric($storeId);

        $publicKey = $this->config->getPublicKey($storeId);
        $privateKey = $this->config->getPrivateKey($storeId);

        if (!empty($publicKey) || !empty($privateKey)) {
            $output->writeln('Already activated');
            return -1;
        }

        list($publicKey, $privateKey, $passphrase) = $this->certificateFactory->create();

        $this->saveKeyData($storeId, $publicKey, $privateKey, $passphrase);

        $this->cleanCache();

        $output->writeln('PUBLIC KEY: ' . PHP_EOL . $publicKey);

        return 0;
    }

    private function cleanCache()
    {
        $types = $this->cacheTypeList->getTypes();

        foreach ($types as $type) {
            $this->cacheTypeList->cleanType($type->getId());
        }
    }

    private function saveKeyData($storeId, $publicKey, $privateKey, $passphrase)
    {
        $scope = ($storeId === null) ? ScopeConfigInterface::SCOPE_TYPE_DEFAULT : ScopeInterface::SCOPE_STORES;

        if ($storeId === null) {
            $storeId = 0;
        }

        $this
            ->writer
            ->save(
                ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_PUBLIC_KEY,
                $this->encryptor->encrypt($publicKey),
                $scope,
                $storeId
            );

        $this
            ->writer
            ->save(
                ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_PRIVATE_KEY,
                $this->encryptor->encrypt($privateKey),
                $scope,
                $storeId
            );

        $this
            ->writer
            ->save(
                ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_PASSPHRASE,
                $this->encryptor->encrypt($passphrase),
                $scope,
                $storeId
            );
    }
}
