<?php

declare(strict_types=1);

namespace Evidente\Aircash\Facade;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Partner\ActivationRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Factory\Partner\ActivationRequestFactoryInterface;
use Evidente\Aircash\Api\Factory\Partner\CertificateFactoryInterface;
use Evidente\Aircash\Builder\Client\AircashPartnerClientBuilder;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Exception\RequestException;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Cache\Type\Config as ConfigType;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\ScopeInterface;
use Exception;

/**
 * @SuppressWarnings(Long)
 * @SuppressWarnings(Coupling)
 */
class PartnerActivationFacade
{
    private $partnerClientBuilder;
    private $activationRequestFactory;
    private $certificateFactory;
    private $writer;
    private $encryptor;
    private $cacheTypeList;

    public function __construct(
        AircashPartnerClientBuilder $partnerClientBuilder,
        ActivationRequestFactoryInterface $activationRequestFactory,
        CertificateFactoryInterface $certificateFactory,
        WriterInterface $writer,
        EncryptorInterface $encryptor,
        TypeListInterface $cacheTypeList
    ) {
        $this->partnerClientBuilder = $partnerClientBuilder;
        $this->activationRequestFactory = $activationRequestFactory;
        $this->certificateFactory = $certificateFactory;
        $this->writer = $writer;
        $this->encryptor = $encryptor;
        $this->cacheTypeList = $cacheTypeList;
    }

    /**
     * @throws Exception
     */
    public function activate(
        bool $testMode,
        int $websiteId,
        string $partnerName,
        string $partnerEmail,
        string $partnerPhone
    ) {
        list($publicKey, $privateKey, $passphrase) = $this->certificateFactory->create();

        try {
            $activationRequest = $this
                ->activationRequestFactory
                ->create($websiteId, $partnerName, $publicKey, $partnerEmail, $partnerPhone);

            $builder = $this->partnerClientBuilder;

            $baseUri = ($testMode)
                ? ConfigInterface::TEST_BASE_URL
                : ConfigInterface::PROD_BASE_URL;

            $builder
                ->withTestMode([$websiteId], $testMode)
                ->withClientBaseUri($baseUri);

            $client = $builder->build();

            $client->activatePartner($activationRequest);

            $currencyCode = CodeRequestInterface::CURRENCY_ID_MAPPING[$activationRequest->getCurrencyID()];

            $this->saveConfigData(
                $testMode,
                $websiteId,
                $activationRequest->getPartnerID(),
                $currencyCode,
                $privateKey,
                $passphrase,
                ActivationRequestInterface::STATUS_PENDING
            );
        } catch (RequestException | InvalidInputException | NoSuchEntityException $exception) {
            // phpcs:disable Magento2.Exceptions.DirectThrow.FoundDirectThrow
            throw new Exception($exception->getMessage(), 0, $exception);
            // phpcs:enable
        }
    }

    private function saveConfigData(
        bool $testMode,
        int $websiteId,
        $partnerId,
        $currencyCode,
        $privateKey,
        $passphrase,
        int $partnerStatus
    ) {
        $scope = ($websiteId === 0) ? ScopeConfigInterface::SCOPE_TYPE_DEFAULT : ScopeInterface::SCOPE_WEBSITES;

        $basePath = ($testMode)
            ? ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_TEST . '/'
            : ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/';

        $this
            ->writer
            ->save(
                $basePath . ConfigInterface::CONFIG_PATH_PARTNER_ID,
                $partnerId,
                $scope,
                $websiteId
            );

        $this
            ->writer
            ->save(
                $basePath . ConfigInterface::CONFIG_PATH_PARTNER_CURRENCY,
                $currencyCode,
                $scope,
                $websiteId
            );

        $this
            ->writer
            ->save(
                $basePath . ConfigInterface::CONFIG_PRIVATE_KEY,
                $this->encryptor->encrypt($privateKey),
                $scope,
                $websiteId
            );

        $this
            ->writer
            ->save(
                $basePath . ConfigInterface::CONFIG_PASSPHRASE,
                $this->encryptor->encrypt($passphrase),
                $scope,
                $websiteId
            );

        $this
            ->writer
            ->save(
                $basePath . ConfigInterface::CONFIG_PATH_PARTNER_STATUS,
                $partnerStatus,
                $scope,
                $websiteId
            );

        $this->cacheTypeList->invalidate(ConfigType::TYPE_IDENTIFIER);
    }
}
