<?php

declare(strict_types=1);

namespace Evidente\Aircash\Facade;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Partner\ActivationRequestInterface;
use Evidente\Aircash\Builder\Client\AircashPartnerClientBuilder;
use Evidente\Aircash\Exception\RequestException;
use Evidente\Aircash\Model\Config\PartnerConfig;
use Magento\Framework\App\Cache\Type\Config as ConfigType;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Store\Model\ScopeInterface;

class PartnerCheckFacade
{
    /**
     * @var AircashPartnerClientBuilder
     */
    private $partnerClientBuilder;

    /**
     * @var WriterInterface
     */
    private $writer;

    /**
     * @var TypeListInterface
     */
    private $cacheTypeList;

    /**
     * @var PartnerConfig
     */
    private $partnerConfig;

    public function __construct(
        AircashPartnerClientBuilder $partnerClientBuilder,
        WriterInterface $writer,
        TypeListInterface $cacheTypeList,
        PartnerConfig $partnerConfig
    ) {
        $this->partnerClientBuilder = $partnerClientBuilder;
        $this->writer = $writer;
        $this->cacheTypeList = $cacheTypeList;
        $this->partnerConfig = $partnerConfig;
    }

    /**
     * @SuppressWarnings(Else)
     */
    public function check(bool $testMode, int $websiteId): array
    {
        $status = $this->partnerConfig->getPartnerStatus($websiteId, $testMode);
        $partnerId = $this->partnerConfig->getPartnerId($websiteId, $testMode);

        $message = null;
        $error = null;

        try {
            $builder = $this->partnerClientBuilder;

            $baseUri = ($testMode)
                ? ConfigInterface::TEST_BASE_URL
                : ConfigInterface::PROD_BASE_URL;

            $builder
                ->withTestMode([$websiteId], $testMode)
                ->withClientBaseUri($baseUri);

            $client = $builder->build();

            $newStatus = $client->checkPartner($partnerId);

            if (!in_array($newStatus, [
                ActivationRequestInterface::STATUS_PENDING,
                ActivationRequestInterface::STATUS_ACTIVATED,
                ActivationRequestInterface::STATUS_REJECTED
            ])) {
                $error = __('Unknown status received');
            } else {
                if ($newStatus === ActivationRequestInterface::STATUS_REJECTED) {
                    $newStatus = ActivationRequestInterface::STATUS_NOT_ACTIVATED;
                    $error = __('Activation rejected');
                }

                if ($newStatus != $status) {
                    $status = $newStatus;
                    $this->saveConfigData($testMode, $websiteId, $status);
                }

                $newStatusText = ActivationRequestInterface::STATUS_LABELS[$newStatus];

                $message = __('Partner Status: %1', $newStatusText);
            }
        } catch (RequestException $exception) {
            $error = __('An error has occurred: %1', $exception->getMessage());
        }

        return [$message, $error];
    }

    private function saveConfigData(bool $testMode, int $websiteId, int $partnerStatus)
    {
        $scope = ($websiteId === 0) ? ScopeConfigInterface::SCOPE_TYPE_DEFAULT : ScopeInterface::SCOPE_WEBSITES;

        $basePath = ($testMode)
            ? ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_TEST . '/'
            : ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/';

        $this
            ->writer
            ->save(
                $basePath . ConfigInterface::CONFIG_PATH_PARTNER_STATUS,
                $partnerStatus,
                $scope,
                $websiteId
            );

        $this->cacheTypeList->invalidate(ConfigType::TYPE_IDENTIFIER);
    }
}
