<?php

declare(strict_types=1);

namespace Evidente\Aircash\Model;

use Evidente\Aircash\Api\AircashClientInterface;
use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Transaction\CancellationRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\CancellationResultInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeResultInterface;
use Evidente\Aircash\Api\Data\Transaction\RefundRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\RefundResultInterface;
use Evidente\Aircash\Api\Factory\HttpClientFactoryInterface;
use Evidente\Aircash\Api\SignatureInterface;
use Evidente\Aircash\Data\Transaction\CancellationResult;
use Evidente\Aircash\Data\Transaction\CodeResult;
use Evidente\Aircash\Data\Transaction\RefundResult;
use Evidente\Aircash\Exception\RequestException;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\RequestOptions;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(Coupling)
 * @SuppressWarnings(Long)
 */
class AircashClient implements AircashClientInterface
{
    /**
     * @var HttpClientFactoryInterface
     */
    private $clientFactory;

    /**
     * @var SignatureInterface
     */
    private $signature;

    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var null|Client
     */
    private $client = null;

    public function __construct(
        HttpClientFactoryInterface $clientFactory,
        SignatureInterface $signature,
        ConfigInterface $config,
        LoggerInterface $logger
    ) {
        $this->clientFactory = $clientFactory;
        $this->signature = $signature;
        $this->config = $config;
        $this->logger = $logger;
    }

    public function getTransactionCode(CodeRequestInterface $request): CodeResultInterface
    {
        $baseErrorMessage = 'Aircash getTransactionCode exception: ';

        $requestData = $request->toArray();

        $websiteId = $request->getWebsiteId();

        $signature = $this
            ->signature
            ->sign(
                $requestData,
                $this->config->getPrivateKey($websiteId),
                $this->config->getPassphrase($websiteId)
            );

        $requestData['Signature'] = $signature;
        $requestData['ValidForPeriod'] = $request->getValidForPeriod() * 60;

        try {
            $response = $this
                ->getClient()
                ->request(
                    'POST',
                    self::GENERATE_PARTNER_CODE_URI,
                    [
                        RequestOptions::JSON => $requestData
                    ]
                );
        } catch (GuzzleException $exception) {
            $baseErrorMessage .= ' request: ' . \json_encode($requestData, JSON_PRETTY_PRINT);

            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s %s, -> trace: %s',
                        $baseErrorMessage,
                        $exception->getMessage(),
                        $exception->getTraceAsString()
                    )
                );

            throw new RequestException(
                __('Request exception'),
                $exception
            );
        }

        $contents = $response->getBody()->getContents();

        $responseData = (array)\json_decode($contents, true);

        $responseData = array_change_key_case($responseData, CASE_LOWER);

        $codeLink = $responseData['codelink'] ?? null;

        if (!is_string($codeLink) || empty(trim($codeLink))) {
            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s -> invalid code link, response: %s',
                        $baseErrorMessage,
                        $contents
                    )
                );

            throw new RequestException(
                __('Invalid code link')
            );
        }

        return new CodeResult($codeLink);
    }

    public function cancelTransaction(CancellationRequestInterface $request): CancellationResultInterface
    {
        $baseErrorMessage = 'Aircash cancelTransaction exception: ';

        $requestData = $request->toArray();

        $websiteId = $request->getWebsiteId();

        $signature = $this
            ->signature
            ->sign(
                $requestData,
                $this->config->getPrivateKey($websiteId),
                $this->config->getPassphrase($websiteId)
            );

        $requestData['Signature'] = $signature;

        try {
            $response = $this
                ->getClient()
                ->request(
                    'POST',
                    self::CANCEL_TRANSACTION_URI,
                    [
                        RequestOptions::JSON => $requestData
                    ]
                );
        } catch (GuzzleException $exception) {
            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s %s, -> trace: %s',
                        $baseErrorMessage,
                        $exception->getMessage(),
                        $exception->getTraceAsString()
                    )
                );

            throw new RequestException(
                __('Request exception'),
                $exception
            );
        }

        $contents = $response->getBody()->getContents();

        $responseData = (array)\json_decode($contents, true);

        $responseData = array_change_key_case($responseData, CASE_LOWER);

        $cancelTransactionID = $responseData['canceltransactionid'] ?? null;

        if (!is_string($cancelTransactionID) || empty(trim($cancelTransactionID))) {
            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s -> invalid transaction ID, response: %s',
                        $baseErrorMessage,
                        $contents
                    )
                );

            throw new RequestException(
                __('Invalid transaction ID')
            );
        }

        return new CancellationResult($cancelTransactionID);
    }

    /**
     * @inheritDoc
     */
    public function refundTransaction(RefundRequestInterface $request): RefundResultInterface
    {
        $baseErrorMessage = 'Aircash refundTransaction exception: ';

        $requestData = $request->toArray();

        $websiteId = $request->getWebsiteId();

        $signature = $this
            ->signature
            ->sign(
                $requestData,
                $this->config->getPrivateKey($websiteId),
                $this->config->getPassphrase($websiteId)
            );

        $requestData['Signature'] = $signature;

        try {
            $response = $this
                ->getClient()
                ->request(
                    'POST',
                    self::REFUND_TRANSACTION_URI,
                    [
                        RequestOptions::JSON => $requestData
                    ]
                );
        } catch (GuzzleException $exception) {
            $baseExceptionMessage = 'Request exception';

            if ($exception instanceof \GuzzleHttp\Exception\RequestException) {
                $errorResponse = (array)\json_decode(
                    (string)$exception->getResponse()->getBody()->getContents(),
                    true
                );

                $errorResponse = array_change_key_case($errorResponse, CASE_LOWER);

                $responseErrorMessage = $errorResponse['errormessage'] ?? null;

                if ($responseErrorMessage !== null) {
                    $baseExceptionMessage .= ': ' . $responseErrorMessage;
                }
            }

            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s %s, -> trace: %s',
                        $baseErrorMessage,
                        $exception->getMessage(),
                        $exception->getTraceAsString()
                    )
                );

            throw new RequestException(
                __($baseExceptionMessage),
                $exception
            );
        }

        $contents = $response->getBody()->getContents();

        $responseData = (array)\json_decode($contents, true);

        $responseData = array_change_key_case($responseData, CASE_LOWER);

        $transactionId = $responseData['transactionid'] ?? null;
        $partnerTransactionId = $responseData['partnertransactionid'] ?? null;
        $refundPartnerTransactionId = $responseData['refundpartnertransactionid'] ?? null;

        if (!is_string($transactionId) ||
            !is_string($partnerTransactionId) ||
            !is_string($refundPartnerTransactionId)
        ) {
            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s -> invalid response: %s',
                        $baseErrorMessage,
                        $contents
                    )
                );

            throw new RequestException(
                __('Invalid response: %1', $contents)
            );
        }

        return new RefundResult(
            $transactionId,
            $partnerTransactionId,
            $refundPartnerTransactionId
        );
    }

    private function getClient(): Client
    {
        if ($this->client === null) {
            $this->client = $this->clientFactory->create();
        }

        return $this->client;
    }
}
