<?php

declare(strict_types=1);

namespace Evidente\Aircash\Model;

use Evidente\Aircash\Api\AircashPartnerClientInterface;
use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Partner\ActivationRequestInterface;
use Evidente\Aircash\Api\SignatureInterface;
use Evidente\Aircash\Exception\RequestException;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\RequestOptions;
use Psr\Log\LoggerInterface;

class AircashPartnerClient implements AircashPartnerClientInterface
{
    /**
     * @var SignatureInterface
     */
    private $signature;

    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var Client
     */
    private $client;

    public function __construct(
        SignatureInterface $signature,
        ConfigInterface $config,
        LoggerInterface $logger,
        Client $client
    ) {
        $this->signature = $signature;
        $this->config = $config;
        $this->logger = $logger;
        $this->client = $client;
    }

    public function activatePartner(ActivationRequestInterface $request): void
    {
        $baseErrorMessage = 'Aircash activatePartner exception: ';

        $requestData = $request->toArray();

        $websiteId = $request->getWebsiteId();

        $signature = $this
            ->signature
            ->sign(
                $requestData,
                $this->config->getPrivateKey($websiteId),
                $this->config->getPassphrase($websiteId)
            );

        $requestData['Signature'] = $signature;

        try {
            $this
                ->getClient()
                ->request(
                    'POST',
                    self::ACTIVATE_PARTNER_URI,
                    [
                        RequestOptions::JSON => $requestData
                    ]
                );
        } catch (GuzzleException $exception) {
            $baseErrorMessage .= $this->getRequestExceptionMessage($exception);

            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s %s, -> trace: %s',
                        $baseErrorMessage,
                        $exception->getMessage(),
                        $exception->getTraceAsString()
                    )
                );

            throw new RequestException(
                __('Request exception: %1', $exception->getMessage()),
                $exception
            );
        }
    }

    /**
     * @inheritDoc
     */
    public function checkPartner(string $partnerId): int
    {
        $baseErrorMessage = 'Aircash checkPartner exception: ';

        try {
            $response = $this
                ->getClient()
                ->request(
                    'GET',
                    self::CHECK_PARTNER_URI,
                    [
                        RequestOptions::QUERY => ['partnerId' => $partnerId]
                    ]
                );
        } catch (GuzzleException $exception) {
            $baseErrorMessage .= $this->getRequestExceptionMessage($exception);

            $this
                ->logger
                ->alert(
                    sprintf(
                        '%s %s, -> trace: %s',
                        $baseErrorMessage,
                        $exception->getMessage(),
                        $exception->getTraceAsString()
                    )
                );

            throw new RequestException(
                __('Request exception: %1', $exception->getMessage()),
                $exception
            );
        }

        $status = str_replace('"', '', (string)$response->getBody()->getContents());

        switch ($status) {
            case 'Pending':
                return ActivationRequestInterface::STATUS_PENDING;
            case 'Activated':
                return ActivationRequestInterface::STATUS_ACTIVATED;
            case 'Rejected':
                return ActivationRequestInterface::STATUS_REJECTED;
            default:
                throw new RequestException(
                    __('Invalid status received: %1', $status)
                );
        }
    }

    private function getClient(): Client
    {
        return $this->client;
    }

    private function getRequestExceptionMessage(GuzzleException $exception)
    {
        $message = '';

        if ($exception instanceof \GuzzleHttp\Exception\RequestException) {
            if ($exception->getRequest()) {
                $message .= ', request body: ' . $exception->getRequest()->getBody()->getContents();
            }

            if ($exception->getResponse()) {
                $message .= ', response body: ' . $exception->getResponse()->getBody()->getContents();
            }
        }

        return $message;
    }
}
