<?php

declare(strict_types=1);

namespace Evidente\Aircash\Model\Payment;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Transaction\ConfirmationResultInterface;
use Evidente\Aircash\Api\TransactionProcessorInterface;
use Evidente\Aircash\Exception\TransactionProcessorException;
use Exception;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\DB\TransactionFactory;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory as OrderCollectionFactory;
use Magento\Sales\Model\ResourceModel\Order\Collection as OrderCollection;
use Magento\Sales\Model\Service\InvoiceService;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(Long)
 * @SuppressWarnings(Coupling)
 */
class TransactionProcessor implements TransactionProcessorInterface
{
    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     * @var OrderCollectionFactory
     */
    private $orderCollectionFactory;

    /**
     * @var TransactionFactory
     */
    private $transactionFactory;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var InvoiceService
     */
    private $invoiceService;

    /**
     * @var LoggerInterface
     */
    private $logger;

    public function __construct(
        ConfigInterface $config,
        OrderCollectionFactory $orderCollectionFactory,
        TransactionFactory $transactionFactory,
        OrderRepositoryInterface $orderRepository,
        InvoiceService $invoiceService,
        LoggerInterface $logger
    ) {
        $this->config = $config;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->transactionFactory = $transactionFactory;
        $this->orderRepository = $orderRepository;
        $this->invoiceService = $invoiceService;
        $this->logger = $logger;
    }

    /**
     * @inheritDoc
     */
    public function process(ConfirmationResultInterface $result): void
    {
        $transactionId = $result->getPartnerTransactionID();

        $prefix = trim((string)$this->config->getTransactionIdPrefix());

        if (!empty($prefix)) {
            $transactionId = str_replace($prefix, '', $transactionId);
        }

        $order = $this->getOrderByTransactionId($transactionId);

        if ($order === null || $order->getPayment()->getCcTransId()) {
            $message = sprintf(
                'Transaction confirmation handler: Order %s not found or already processed',
                $transactionId
            );

            $this->logger->alert($message);

            throw new TransactionProcessorException(
                __('Order not found or already processed')
            );
        }

        $orderGrandTotal = number_format((float)$order->getGrandTotal(), 2, '.', '');
        $receivedGrandTotal = number_format((float)$result->getAmount(), 2, '.', '');

        if ($orderGrandTotal !== $receivedGrandTotal) {
            $message = sprintf(
                'Aircash amount mismatch for order %s -> expected: %s, received: %s',
                $transactionId,
                $orderGrandTotal,
                $receivedGrandTotal
            );

            $this->logger->alert($message);

            throw new TransactionProcessorException(
                __('Order grand total amount mismatch')
            );
        }

        $order
            ->getPayment()
            ->setCcTransId($result->getAircashTransactionID());

        $this->orderRepository->save($order);

        try {
            $invoice = $this->invoiceService->prepareInvoice($order);

            $invoice->setTransactionId($result->getAircashTransactionID());

            $invoice->setRequestedCaptureCase(Invoice::CAPTURE_ONLINE);

            $invoice->register();

            $this->transactionFactory
                ->create()
                ->addObject($invoice)
                ->addObject($order)
                ->save();
        } catch (LocalizedException | Exception $exception) {
            $message = sprintf(
                'Aircash failed to create invoice for order %s -> %s',
                $transactionId,
                $exception->getMessage()
            );

            $this->logger->alert($message);

            throw new TransactionProcessorException(
                __('Failed to create invoice')
            );
        }
    }

    private function getOrderByTransactionId(string $transactionId): ?OrderInterface
    {
        /**
         * @var OrderCollection
         */
        $collection = $this->orderCollectionFactory->create();

        $collection
            ->addFieldToFilter('increment_id', $transactionId);

        return ($collection->count())
            ? $this->orderRepository->get($collection->getFirstItem()->getId())
            : null;
    }
}
