<?php

declare(strict_types=1);

namespace Evidente\Aircash\Model;

use Evidente\Aircash\Api\SignatureInterface;
use Evidente\Aircash\Api\SignatureStringFactoryInterface;
use Evidente\Aircash\Exception\InvalidSignatureException;

/**
 * @SuppressWarnings(Long)
 */
class Signature implements SignatureInterface
{
    /**
     * @var SignatureStringFactoryInterface
     */
    private $signatureStringFactory;

    public function __construct(
        SignatureStringFactoryInterface $signatureStringFactory
    ) {
        $this->signatureStringFactory = $signatureStringFactory;
    }

    public function sign(array $input, $privateKey, ?string $passphrase = null): string
    {
        $stringToSign = $this->inputToString($input);

        if ($passphrase === null) {
            $passphrase = '';
        }

        $privateKey = openssl_get_privatekey($privateKey, $passphrase);

        openssl_sign(
            $stringToSign,
            $signature,
            $privateKey,
            OPENSSL_ALGO_SHA256
        );

        return base64_encode($signature);
    }

    /**
     * @inheritDoc
     */
    public function validate(array $input, string $givenSignature, $publicKey): void
    {
        $stringToVerify = $this->inputToString($input);

        $publicKey = openssl_pkey_get_public($publicKey);

        // phpcs:ignore Magento2.Functions.DiscouragedFunction
        $decodedSignature = base64_decode($givenSignature);

        if (openssl_verify(
            $stringToVerify,
            $decodedSignature,
            $publicKey,
            OPENSSL_ALGO_SHA256
        ) !== 1) {
            throw new InvalidSignatureException(
                __(
                    'Invalid signature: %1 for input %2',
                    $givenSignature,
                    $stringToVerify
                )
            );
        }
    }

    private function inputToString(array $input): string
    {
        return $this->signatureStringFactory->createFromArray($input);
    }
}
