<?php

declare(strict_types=1);

namespace Evidente\Aircash\Rest;

use DateTimeZone;
use Evidente\Aircash\Api\AircashClientInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeResultInterface;
use Evidente\Aircash\Api\Factory\Transaction\CodeRequestFactoryInterface;
use Evidente\Aircash\Api\Rest\CheckoutDataProviderInterface;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Exception\PaymentExpiredException;
use Evidente\Aircash\Exception\RequestException;
use Magento\Checkout\Model\Session;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\OrderRepositoryInterface;
use Psr\Log\LoggerInterface;
use DateTime;

/**
 * @SuppressWarnings(Long)
 * @SuppressWarnings(Coupling)
 */
class CheckoutSessionDataProvider implements CheckoutDataProviderInterface
{
    /**
     * @var null|CodeRequestInterface
     */
    private $lastCodeRequest;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var Session
     */
    private $session;

    /**
     * @var CodeRequestFactoryInterface
     */
    private $codeRequestFactory;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var AircashClientInterface
     */
    private $aircashClient;

    public function __construct(
        OrderRepositoryInterface $orderRepository,
        Session $session,
        CodeRequestFactoryInterface $codeRequestFactory,
        AircashClientInterface $aircashClient,
        LoggerInterface $logger
    ) {
        $this->orderRepository = $orderRepository;
        $this->session = $session;
        $this->codeRequestFactory = $codeRequestFactory;
        $this->aircashClient = $aircashClient;
        $this->logger = $logger;
    }

    public function getLastCodeRequest(): ?CodeRequestInterface
    {
        return $this->lastCodeRequest;
    }

    public function getCode(): CodeResultInterface
    {
        $this->lastCodeRequest = null;

        $placedOrderId = $this->session->getLastOrderId();

        if (!$placedOrderId) {
            throw new LocalizedException(__('Order not found'));
        }

        try {
            $order = $this->orderRepository->get($placedOrderId);
        } catch (NoSuchEntityException $exception) {
            throw new LocalizedException(__('Order not found'));
        }

        try {
            $request = $this
                ->codeRequestFactory
                ->createFromOrder($order);
        } catch (InvalidInputException $exception) {
            $this
                ->logger
                ->alert(
                    sprintf(
                        'Aircash checkout session data provider exception on order #%s: %s',
                        $order->getIncrementId(),
                        $exception->getMessage()
                    )
                );

            throw new LocalizedException(__('Invalid input'));
        }

        $payment = $order->getPayment();

        if (!$payment) {
            throw new LocalizedException(__('Payment not found'));
        }

        $expires = $payment->getData(self::PAYMENT_FIELD_EXPIRES);

        if ($expires) {
            $expires = new DateTime($expires, new DateTimeZone('UTC'));
            $now = new DateTime('now', new DateTimeZone('UTC'));

            if ($now > $expires) {
                throw new PaymentExpiredException(__('Payment expired'));
            }
        }

        try {
            $result = $this->aircashClient->getTransactionCode($request);
            $this->lastCodeRequest = $request;

            if (!$expires) {
                $now = new DateTime('now', new DateTimeZone('UTC'));
                $now->modify('+' . $request->getValidForPeriod() . ' minutes');
                $payment->setData(self::PAYMENT_FIELD_EXPIRES, $now->format('Y-m-d H:i:s'));
                $this->orderRepository->save($order);
            }

            return $result;
        } catch (RequestException $exception) {
            throw new LocalizedException(__('Request exception'));
        }
    }
}
