<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Controller\Adminhtml\Activation;

use Evidente\Aircash\Controller\Adminhtml\Activation\Activate;
use Evidente\Aircash\Facade\PartnerActivationFacade;
use Magento\Framework\Exception\NotFoundException;
use PHPUnit\Framework\MockObject\MockObject;
use Exception;

/**
 * @SuppressWarnings(PHPMD)
 */
class ActivateTest extends AbstractControllerTest
{
    /**
     * @var PartnerActivationFacade|MockObject
     */
    private $partnerActivationFacade;

    /**
     * @var Activate
     */
    private $subject;

    protected function setUp(): void
    {
        parent::setUp();

        $this->partnerActivationFacade = $this->createMock(PartnerActivationFacade::class);

        $this->subject = new Activate(
            $this->context,
            $this->partnerActivationFacade
        );
    }

    /**
     * @throws NotFoundException
     */
    public function testExecuteShouldFailBecauseOfInvalidInput()
    {
        // given
        $expectedRedirectUrl = '/';

        $expectedMessages = [
            __('Invalid partner name'),
            __('Invalid partner email'),
            __('Invalid partner phone number'),
        ];

        $expectedMessageText = implode(', ', $expectedMessages);

        $this->request->setParam(Activate::PARAM_PARTNER_NAME, '');
        $this->request->setParam(Activate::PARAM_PARTNER_EMAIL, 'email');
        $this->request->setParam(Activate::PARAM_PARTNER_PHONE, '');

        $this
            ->helper
            ->expects($this->once())
            ->method('getUrl')
            ->willReturn($expectedRedirectUrl);

        $this
            ->messageManager
            ->expects($this->once())
            ->method('addErrorMessage')
            ->with($expectedMessageText);

        $this
            ->redirect
            ->expects($this->once())
            ->method('getRefererUrl')
            ->willReturn($expectedRedirectUrl);

        $this
            ->response
            ->expects($this->once())
            ->method('setRedirect')
            ->with($expectedRedirectUrl, 302);

        // when, then
        $this->subject->execute();
    }

    /**
     * @throws NotFoundException
     */
    public function testExecuteShouldFailBecauseOfActivationFacadeFailure()
    {
        // given
        $expectedRedirectUrl = '/';
        $expectedPartnerName = 'Partner Name';
        $expectedEmail = 'email@example.com';
        $expectedPhone = 'phone';

        $this
            ->request
            ->method('getParam')
            ->withConsecutive(
                [Activate::PARAM_PARTNER_NAME],
                [Activate::PARAM_PARTNER_EMAIL],
                [Activate::PARAM_PARTNER_PHONE]
            )
            ->willReturnOnConsecutiveCalls($expectedPartnerName, $expectedEmail, $expectedPhone);

        $this
            ->partnerActivationFacade
            ->method('activate')
            ->willThrowException(
                new Exception()
            );

        $this
            ->helper
            ->expects($this->once())
            ->method('getUrl')
            ->willReturn($expectedRedirectUrl);

        $this
            ->messageManager
            ->expects($this->once())
            ->method('addErrorMessage')
            ->with(__('Activation failed.'));

        $this
            ->redirect
            ->expects($this->once())
            ->method('getRefererUrl')
            ->willReturn($expectedRedirectUrl);

        $this
            ->response
            ->expects($this->once())
            ->method('setRedirect')
            ->with($expectedRedirectUrl, 302);

        // when, then
        $this->subject->execute();
    }

    /**
     * @throws NotFoundException
     */
    public function testExecute()
    {
        // given
        $expectedRedirectUrl = 'adminhtml/system_config/edit';
        $expectedPartnerName = 'Partner Name';
        $expectedEmail = 'email@example.com';
        $expectedPhone = 'phone';
        $expectedWebsiteId = 1;

        $this
            ->request
            ->method('getParam')
            ->withConsecutive(
                [Activate::PARAM_PARTNER_NAME],
                [Activate::PARAM_PARTNER_EMAIL],
                [Activate::PARAM_PARTNER_PHONE],
                [Activate::PARAM_WEBSITE]
            )
            ->willReturnOnConsecutiveCalls($expectedPartnerName, $expectedEmail, $expectedPhone, $expectedWebsiteId);

        $this
            ->helper
            ->expects($this->atLeastOnce())
            ->method('getUrl')
            ->willReturn($expectedRedirectUrl);

        $this
            ->messageManager
            ->expects($this->once())
            ->method('addSuccessMessage')
            ->with(__('Partner activation successfully submitted.'));

        $this
            ->response
            ->expects($this->once())
            ->method('setRedirect')
            ->with($expectedRedirectUrl, 302);

        // when, then
        $this->subject->execute();
    }
}
