<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Data\Partner;

use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Factory\Partner\ActivationRequestFactory;
use Magento\Directory\Model\Currency;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Store\Model\Website;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use ReflectionException;

class ActivationRequestFactoryTest extends TestCase
{
    /**
     * @var PriceCurrencyInterface|MockObject
     */
    private $priceCurrency;

    /**
     * @var StoreManagerInterface|MockObject
     */
    private $storeManager;

    /**
     * @var Store|MockObject
     */
    private $store;

    /**
     * @var Website|MockObject
     */
    private $website;

    /**
     * @var Currency
     */
    private $currency;

    /**
     * @var ActivationRequestFactory
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->priceCurrency = $this->createMock(PriceCurrencyInterface::class);
        $this->storeManager = $this->createMock(StoreManagerInterface::class);

        $this->store = $this->createMock(Store::class);
        $this->website = $this->createMock(Website::class);

        $this->storeManager->method('getWebsite')->willReturn($this->website);
        $this->storeManager->method('getStore')->willReturn($this->store);

        $this->currency = $this->createMock(Currency::class);

        $this->priceCurrency->method('getCurrency')->willReturn($this->currency);

        $this->subject = new ActivationRequestFactory(
            $this->priceCurrency,
            $this->storeManager
        );
    }

    /**
     * @throws NoSuchEntityException
     * @throws InvalidInputException
     */
    public function testCreate()
    {
        // given
        $this->currency->method('getCode')->willReturn(CodeRequestInterface::CURRENCY_EUR);

        $expectedBaseUrl = 'https://example.com/';
        $expectedFullUrl = $expectedBaseUrl . 'rest/V1/payment/aircash/confirm-transaction';
        $expectedStoreId = 1;
        $expectedPartnerName = 'partner_name';
        $expectedPublicKey = 'public_key';
        $expectedContactEmail = 'email@example.com';
        $expectedPhoneNumber = '5554444';
        $expectedCurrencyID = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];

        $this->store->method('getBaseUrl')->willReturn($expectedBaseUrl);

        // when
        $request = $this->subject->create(
            $expectedStoreId,
            $expectedPartnerName,
            $expectedPublicKey,
            $expectedContactEmail,
            $expectedPhoneNumber
        );

        $this->assertSame($expectedPhoneNumber, $request->getPhoneNumber());
        $this->assertSame($expectedContactEmail, $request->getContactEmail());
        $this->assertSame($expectedFullUrl, $request->getConfirmTransactionURL());
        $this->assertSame($expectedCurrencyID, $request->getCurrencyID());
        $this->assertSame($expectedPublicKey, $request->getPublicKey());
        $this->assertSame($expectedPartnerName, $request->getPartnerName());
        $this->assertNotEmpty($request->getPartnerID());
        $this->assertNotEmpty($request->getPlatformID());
    }

    /**
     * @throws NoSuchEntityException
     * @throws InvalidInputException
     */
    public function testCreateShouldFailOnUnsupportedCurrency()
    {
        // given
        $this->currency->method('getCode')->willReturn('unsupported_currency');

        $expectedStoreId = 1;
        $expectedPartnerName = 'partner_name';
        $expectedPublicKey = 'public_key';
        $expectedContactEmail = 'email@example.com';
        $expectedPhoneNumber = '5554444';

        $this->expectExceptionMessage('Unsupported currency');

        // then
        $this->subject->create(
            $expectedStoreId,
            $expectedPartnerName,
            $expectedPublicKey,
            $expectedContactEmail,
            $expectedPhoneNumber
        );
    }
}
