<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Facade;

use Evidente\Aircash\Api\AircashPartnerClientInterface;
use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Partner\ActivationRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Factory\Partner\ActivationRequestFactoryInterface;
use Evidente\Aircash\Api\Factory\Partner\CertificateFactoryInterface;
use Evidente\Aircash\Builder\Client\AircashPartnerClientBuilder;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Exception\RequestException;
use Evidente\Aircash\Facade\PartnerActivationFacade;
use Exception;
use Magento\Framework\App\Cache\Type\Config as ConfigType;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Store\Model\ScopeInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class PartnerActivationFacadeTest extends TestCase
{
    /**
     * @var AircashPartnerClientBuilder|MockObject
     */
    private $partnerClientBuilder;

    /**
     * @var ActivationRequestFactoryInterface|MockObject
     */
    private $activationRequestFactory;

    /**
     * @var CertificateFactoryInterface
     */
    private $certificateFactory;

    /**
     * @var AircashPartnerClientInterface|MockObject
     */
    private $aircashPartnerClient;

    /**
     * @var WriterInterface|MockObject
     */
    private $writer;

    /**
     * @var EncryptorInterface|MockObject
     */
    private $encryptor;

    /**
     * @var TypeListInterface|MockObject
     */
    private $cacheTypeList;

    /**
     * @var ActivationRequestInterface|MockObject
     */
    private $activationRequest;

    /**
     * @var PartnerActivationFacade
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->partnerClientBuilder = $this->createMock(AircashPartnerClientBuilder::class);
        $this->activationRequestFactory = $this->createMock(ActivationRequestFactoryInterface::class);
        $this->certificateFactory = $this->createMock(CertificateFactoryInterface::class);

        $this->aircashPartnerClient = $this->createMock(AircashPartnerClientInterface::class);

        $this->partnerClientBuilder->method('withTestMode')->willReturn($this->partnerClientBuilder);
        $this->partnerClientBuilder->method('build')->willReturn($this->aircashPartnerClient);

        $this->writer = $this->createMock(WriterInterface::class);
        $this->encryptor = $this->createMock(EncryptorInterface::class);
        $this->cacheTypeList = $this->createMock(TypeListInterface::class);

        $this->activationRequest = $this->createMock(ActivationRequestInterface::class);

        $this->subject = new PartnerActivationFacade(
            $this->partnerClientBuilder,
            $this->activationRequestFactory,
            $this->certificateFactory,
            $this->writer,
            $this->encryptor,
            $this->cacheTypeList
        );
    }

    /**
     * @throws Exception
     */
    public function testActivateShouldFailBecauseOfInvalidInput()
    {
        // given
        $expectedPublicKey = 'public_key';
        $expectedPrivateKey = 'private_key';
        $expectedPassphrase = 'passphrase';
        $expectedTestMode = false;

        $this
            ->certificateFactory
            ->method('create')
            ->willReturn([$expectedPublicKey, $expectedPrivateKey, $expectedPassphrase]);

        $this
            ->activationRequestFactory
            ->expects($this->once())
            ->method('create')
            ->willThrowException(new InvalidInputException(__('Invalid input')));

        $this->expectExceptionMessage('Invalid input');

        // when, then
        $this
            ->subject
            ->activate($expectedTestMode, 1, 'Partner Name', 'email@example.com', 'phone');
    }

    /**
     * @throws Exception
     */
    public function testActivateShouldFailBecauseOfRequestException()
    {
        // given
        $expectedPublicKey = 'public_key';
        $expectedPrivateKey = 'private_key';
        $expectedPassphrase = 'passphrase';
        $expectedTestMode = false;

        $this
            ->certificateFactory
            ->method('create')
            ->willReturn([$expectedPublicKey, $expectedPrivateKey, $expectedPassphrase]);

        $this
            ->activationRequestFactory
            ->expects($this->once())
            ->method('create')
            ->willReturn($this->activationRequest);

        $this
            ->aircashPartnerClient
            ->expects($this->once())
            ->method('activatePartner')
            ->willThrowException(new RequestException(__('Request exception')));

        $this->expectExceptionMessage('Request exception');

        // when, then
        $this
            ->subject
            ->activate($expectedTestMode, 1, 'Partner Name', 'email@example.com', 'phone');
    }

    /**
     * @throws Exception
     */
    public function testActivate()
    {
        // given
        $expectedPartnerName = 'Partner Name';
        $expectedEmail = 'email@example.com';
        $expectedPhone = 'phone';
        $expectedPartnerId = 'partner_id';
        $expectedWebsiteId = 1;
        $expectedCurrency = 'HRK';
        $expectedTestMode = false;

        $expectedPublicKey = 'public_key';
        $expectedPrivateKey = 'private_key';
        $expectedPassphrase = 'passphrase';

        $expectedScope = ScopeInterface::SCOPE_WEBSITES;

        $this
            ->certificateFactory
            ->method('create')
            ->willReturn([$expectedPublicKey, $expectedPrivateKey, $expectedPassphrase]);

        $this
            ->activationRequest
            ->method('getPartnerID')
            ->willReturn($expectedPartnerId);

        $this
            ->activationRequest
            ->method('getCurrencyID')
            ->willReturn(CodeRequestInterface::CURRENCY_MAPPING[$expectedCurrency]);

        $this
            ->activationRequestFactory
            ->expects($this->once())
            ->method('create')
            ->willReturn($this->activationRequest);

        $this
            ->encryptor
            ->expects($this->exactly(2))
            ->method('encrypt')
            ->withConsecutive(
                [$expectedPrivateKey],
                [$expectedPassphrase]
            )
            ->willReturnOnConsecutiveCalls($expectedPrivateKey, $expectedPassphrase);

        $basePath = ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/';

        $this
            ->writer
            ->expects($this->exactly(5))
            ->method('save')
            ->withConsecutive(
                [
                    $basePath . ConfigInterface::CONFIG_PATH_PARTNER_ID,
                    $expectedPartnerId,
                    $expectedScope,
                    $expectedWebsiteId
                ],
                [
                    $basePath . ConfigInterface::CONFIG_PATH_PARTNER_CURRENCY,
                    $expectedCurrency,
                    $expectedScope,
                    $expectedWebsiteId
                ],
                [
                    $basePath . ConfigInterface::CONFIG_PRIVATE_KEY,
                    $expectedPrivateKey,
                    $expectedScope,
                    $expectedWebsiteId
                ],
                [
                    $basePath . ConfigInterface::CONFIG_PASSPHRASE,
                    $expectedPassphrase,
                    $expectedScope,
                    $expectedWebsiteId
                ],
                [
                    $basePath . ConfigInterface::CONFIG_PATH_PARTNER_STATUS,
                    ActivationRequestInterface::STATUS_PENDING,
                    $expectedScope,
                    $expectedWebsiteId
                ]
            );

        $this
            ->cacheTypeList
            ->expects($this->once())
            ->method('invalidate')
            ->with(ConfigType::TYPE_IDENTIFIER);

        // when, then
        $this
            ->subject
            ->activate($expectedTestMode, $expectedWebsiteId, $expectedPartnerName, $expectedEmail, $expectedPhone);
    }
}
