<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Facade;

use Evidente\Aircash\Api\AircashPartnerClientInterface;
use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Partner\ActivationRequestInterface;
use Evidente\Aircash\Builder\Client\AircashPartnerClientBuilder;
use Evidente\Aircash\Exception\RequestException;
use Evidente\Aircash\Facade\PartnerCheckFacade;
use Evidente\Aircash\Model\Config\PartnerConfig;
use Exception;
use Magento\Framework\App\Cache\Type\Config as ConfigType;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\Phrase;
use Magento\Store\Model\ScopeInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class PartnerCheckFacadeTest extends TestCase
{
    /**
     * @var AircashPartnerClientBuilder|MockObject
     */
    private $partnerClientBuilder;

    /**
     * @var AircashPartnerClientInterface|MockObject
     */
    private $aircashPartnerClient;

    /**
     * @var WriterInterface|MockObject
     */
    private $writer;

    /**
     * @var TypeListInterface|MockObject
     */
    private $cacheTypeList;

    /**
     * @var PartnerConfig|MockObject
     */
    private $partnerConfig;

    /**
     * @var PartnerCheckFacade
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->aircashPartnerClient = $this->createMock(AircashPartnerClientInterface::class);

        $this->partnerClientBuilder = $this->createMock(AircashPartnerClientBuilder::class);

        $this->partnerClientBuilder->method('withTestMode')->willReturn($this->partnerClientBuilder);
        $this->partnerClientBuilder->method('build')->willReturn($this->aircashPartnerClient);

        $this->writer = $this->createMock(WriterInterface::class);
        $this->cacheTypeList = $this->createMock(TypeListInterface::class);
        $this->partnerConfig = $this->createMock(PartnerConfig::class);

        $this->subject = new PartnerCheckFacade(
            $this->partnerClientBuilder,
            $this->writer,
            $this->cacheTypeList,
            $this->partnerConfig
        );
    }

    /**
     * @throws Exception
     */
    public function testActivateShouldFailBecauseOfRequestException()
    {
        // given
        $expectedWebsiteId = 1;
        $expectedPartnerId = 'partner_id';
        $expectedTestMode = false;

        $this->partnerConfig->method('getPartnerId')->willReturn($expectedPartnerId);

        $this
            ->aircashPartnerClient
            ->expects($this->once())
            ->method('checkPartner')
            ->willThrowException(new RequestException(__('Request exception')));

        // when
        list($message, $error) =  $this->subject->check($expectedTestMode, $expectedWebsiteId);

        if ($error instanceof Phrase) {
            $error = $error->__toString();
        }

        // then
        $this->assertNull($message);
        $this->assertNotFalse(strpos($error, 'Request exception'));
    }

    /**
     * @throws Exception
     */
    public function testActivateShouldFailBecauseOfInvalidStatus()
    {
        // given
        $expectedWebsiteId = 1;
        $expectedPartnerId = 'partner_id';
        $expectedStatus = 100;
        $expectedTestMode = false;

        $this->partnerConfig->method('getPartnerId')->willReturn($expectedPartnerId);

        $this
            ->aircashPartnerClient
            ->expects($this->once())
            ->method('checkPartner')
            ->willReturn($expectedStatus);

        // when
        list($message, $error) =  $this->subject->check($expectedTestMode, $expectedWebsiteId);

        if ($error instanceof Phrase) {
            $error = $error->__toString();
        }

        // then
        $this->assertNull($message);
        $this->assertNotFalse(strpos($error, 'Unknown status received'));
    }

    /**
     * @throws Exception
     */
    public function testActivateShouldResetRejectedStatus()
    {
        // given
        $expectedWebsiteId = 1;
        $expectedPartnerId = 'partner_id';
        $oldStatus = ActivationRequestInterface::STATUS_PENDING;
        $expectedStatus = ActivationRequestInterface::STATUS_REJECTED;
        $expectedMessage = __(
            'Partner Status: %1',
            ActivationRequestInterface::STATUS_LABELS[ActivationRequestInterface::STATUS_NOT_ACTIVATED]
        )->__toString();
        $expectedTestMode = false;

        $expectedScope = ScopeInterface::SCOPE_WEBSITES;

        $this->partnerConfig->method('getPartnerId')->willReturn($expectedPartnerId);
        $this->partnerConfig->method('getPartnerStatus')->willReturn($oldStatus);

        $this
            ->aircashPartnerClient
            ->expects($this->once())
            ->method('checkPartner')
            ->willReturn($expectedStatus);

        $basePath = ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/';

        $this
            ->writer
            ->expects($this->exactly(1))
            ->method('save')
            ->withConsecutive(
                [
                    $basePath . ConfigInterface::CONFIG_PATH_PARTNER_STATUS,
                    ActivationRequestInterface::STATUS_NOT_ACTIVATED,
                    $expectedScope,
                    $expectedWebsiteId
                ]
            );

        $this
            ->cacheTypeList
            ->expects($this->once())
            ->method('invalidate')
            ->with(ConfigType::TYPE_IDENTIFIER);

        // when
        list($message, $error) =  $this->subject->check($expectedTestMode, $expectedWebsiteId);

        // then
        if ($message instanceof Phrase) {
            $message = $message->__toString();
        }

        if ($error instanceof Phrase) {
            $error = $error->__toString();
        }

        $this->assertSame($expectedMessage, $message);
        $this->assertNotFalse(strpos($error, 'Activation rejected'));
    }

    /**
     * @throws Exception
     */
    public function testActivate()
    {
        // given
        $expectedWebsiteId = 1;
        $expectedPartnerId = 'partner_id';
        $oldStatus = ActivationRequestInterface::STATUS_PENDING;
        $expectedStatus = ActivationRequestInterface::STATUS_ACTIVATED;
        $expectedMessage = __(
            'Partner Status: %1',
            ActivationRequestInterface::STATUS_LABELS[ActivationRequestInterface::STATUS_ACTIVATED]
        )->__toString();
        $expectedTestMode = false;

        $expectedScope = ScopeInterface::SCOPE_WEBSITES;

        $this->partnerConfig->method('getPartnerId')->willReturn($expectedPartnerId);
        $this->partnerConfig->method('getPartnerStatus')->willReturn($oldStatus);

        $this
            ->aircashPartnerClient
            ->expects($this->once())
            ->method('checkPartner')
            ->willReturn($expectedStatus);

        $basePath = ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/';

        $this
            ->writer
            ->expects($this->exactly(1))
            ->method('save')
            ->withConsecutive(
                [
                    $basePath . ConfigInterface::CONFIG_PATH_PARTNER_STATUS,
                    ActivationRequestInterface::STATUS_ACTIVATED,
                    $expectedScope,
                    $expectedWebsiteId
                ]
            );

        $this
            ->cacheTypeList
            ->expects($this->once())
            ->method('invalidate')
            ->with(ConfigType::TYPE_IDENTIFIER);

        // when
        list($message, $error) =  $this->subject->check($expectedTestMode, $expectedWebsiteId);

        // then
        if ($message instanceof Phrase) {
            $message = $message->__toString();
        }

        if ($error instanceof Phrase) {
            $error = $error->__toString();
        }

        $this->assertSame($expectedMessage, $message);
        $this->assertNull($error);
    }
}
