<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Factory\Transaction;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Factory\Transaction\CodeRequestFactoryInterface;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Factory\Transaction\CodeRequestFactory;
use Magento\Sales\Model\Order;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class CodeRequestFactoryTest extends TestCase
{
    /**
     * @var Order|MockObject
     */
    private $order;

    /**
     * @var ConfigInterface|MockObject
     */
    private $config;

    /**
     * @var StoreManagerInterface|MockObject
     */
    private $storeManager;

    /**
     * @var Store|MockObject
     */
    private $store;

    /**
     * @var CodeRequestFactoryInterface
     */
    private $subject;

    private $storeId = 1;
    private $websiteId = 1;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->order = $this->createMock(Order::class);
        $this->config = $this->createMock(ConfigInterface::class);
        $this->storeManager = $this->createMock(StoreManagerInterface::class);

        $this->store = $this->createMock(Store::class);
        $this->store->method('getId')->willReturn($this->storeId);
        $this->store->method('getWebsiteId')->willReturn($this->websiteId);

        $this->storeManager->method('getStore')->willReturn($this->store);

        $this->subject = new CodeRequestFactory($this->config, $this->storeManager);
    }

    /**
     * @throws InvalidInputException
     */
    public function testCreateFromOrder()
    {
        // given
        $expectedIncrementId = '000000001';
        $expectedGrandTotal = 125.00;

        $expectedPartnerId = 'partner_id';
        $expectedLocationId = 'location_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];

        $this->order->method('getOrderCurrencyCode')->willReturn(CodeRequestInterface::CURRENCY_EUR);
        $this->order->method('getIncrementId')->willReturn($expectedIncrementId);
        $this->order->method('getGrandTotal')->willReturn($expectedGrandTotal);

        $this->config->method('getPartnerID')->willReturn($expectedPartnerId);
        $this->config->method('getLocationID')->willReturn($expectedLocationId);

        // when
        $model = $this->subject->createFromOrder($this->order);

        // then
        $this->assertSame($expectedPartnerId, $model->getPartnerID());
        $this->assertSame($expectedGrandTotal, $model->getAmount());
        $this->assertSame($expectedCurrency, $model->getCurrencyID());
        $this->assertSame($expectedIncrementId, $model->getPartnerTransactionID());
        $this->assertSame('Order-' . $expectedIncrementId, $model->getDescription());
        $this->assertSame($expectedLocationId, $model->getLocationID());
    }

    /**
     * @throws InvalidInputException
     */
    public function testCreateFromOrderWithPrefix()
    {
        // given
        $transactionIdPrefix = 'prefix';
        $expectedIncrementId = '000000001';
        $expectedTransactionId = $transactionIdPrefix . $expectedIncrementId;
        $expectedGrandTotal = 125.00;

        $expectedPartnerId = 'partner_id';
        $expectedLocationId = 'location_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];

        $this->order->method('getOrderCurrencyCode')->willReturn(CodeRequestInterface::CURRENCY_EUR);
        $this->order->method('getIncrementId')->willReturn($expectedIncrementId);
        $this->order->method('getGrandTotal')->willReturn($expectedGrandTotal);

        $this->config->method('getPartnerID')->willReturn($expectedPartnerId);
        $this->config->method('getLocationID')->willReturn($expectedLocationId);
        $this->config->method('getTransactionIdPrefix')->willReturn($transactionIdPrefix);

        // when
        $model = $this->subject->createFromOrder($this->order);

        // then
        $this->assertSame($expectedPartnerId, $model->getPartnerID());
        $this->assertSame($expectedGrandTotal, $model->getAmount());
        $this->assertSame($expectedCurrency, $model->getCurrencyID());
        $this->assertSame($expectedTransactionId, $model->getPartnerTransactionID());
        $this->assertSame('Order-' . $expectedTransactionId, $model->getDescription());
        $this->assertSame($expectedLocationId, $model->getLocationID());
    }

    /**
     * @throws InvalidInputException
     */
    public function testCreateFromOrderShouldFailBecauseOfInvalidCurrency()
    {
        // given
        $this->order->method('getOrderCurrencyCode')->willReturn('Invalid currency');
        $this->expectExceptionMessage('Invalid currency');

        // then
        $this->subject->createFromOrder($this->order);
    }

    /**
     * @throws InvalidInputException
     */
    public function testCreateFromOrderShouldFailBecauseOfMissingTransactionId()
    {
        // given
        $this->order->method('getOrderCurrencyCode')->willReturn(CodeRequestInterface::CURRENCY_EUR);
        $this->expectExceptionMessage('Increment order ID is required for transaction ID');

        // then
        $this->subject->createFromOrder($this->order);
    }
}
