<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Factory\Transaction;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\ConfirmationResultInterface;
use Evidente\Aircash\Api\Factory\Transaction\ConfirmationResultFactoryInterface;
use Evidente\Aircash\Api\SignatureInterface;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Exception\InvalidSignatureException;
use Evidente\Aircash\Factory\Transaction\ConfirmationResultFactory;
use Magento\Framework\App\Request\Http;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Psr\Log\LoggerInterface;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class ConfirmationResultFactoryTest extends TestCase
{
    /**
     * @var Http|MockObject
     */
    private $request;

    /**
     * @var SignatureInterface|MockObject
     */
    private $signature;

    /**
     * @var ConfigInterface|MockObject
     */
    private $config;

    /**
     * @var LoggerInterface|MockObject
     */
    private $logger;

    /**
     * @var ConfirmationResultFactoryInterface
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->request = $this->createMock(Http::class);
        $this->signature = $this->createMock(SignatureInterface::class);
        $this->config = $this->createMock(ConfigInterface::class);
        $this->logger = $this->createMock(LoggerInterface::class);

        $this->subject = new ConfirmationResultFactory(
            $this->request,
            $this->signature,
            $this->config,
            $this->logger
        );
    }

    /**
     * @throws InvalidSignatureException
     * @throws InvalidInputException
     */
    public function testCreateFromRequest()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $this->request->method('getContent')->willReturn(\json_encode($expectedInput));

        $this->logger->expects($this->never())->method('alert');

        // when
        $model = $this->subject->createFromRequest();

        // then
        $this->assertSame($expectedPartnerTransactionId, $model->getPartnerTransactionID());
        $this->assertSame($expectedCurrency, $model->getCurrencyID());
        $this->assertSame($expectedAmount, $model->getAmount());
        $this->assertSame($expectedPartnerId, $model->getPartnerID());
        $this->assertSame($expectedAircashTransactionId, $model->getAircashTransactionID());
        $this->assertSame($expectedSignature, $model->getSignature());
    }

    /**
     * @throws InvalidSignatureException
     * @throws InvalidInputException
     */
    public function testCreateFromRequestShouldFailOnBadInput()
    {
        // given
        $expectedInput = [];

        $this->request->method('getContent')->willReturn(\json_encode($expectedInput));

        $this->logger->expects($this->once())->method('alert');

        $this->expectExceptionMessage('Invalid input');

        // then
        $this->subject->createFromRequest();
    }

    /**
     * @throws InvalidSignatureException
     * @throws InvalidInputException
     */
    public function testCreateFromRequestShouldFailOnBadSignature()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $this->request->method('getContent')->willReturn(\json_encode($expectedInput));

        $this->logger->expects($this->once())->method('alert');

        $this
            ->signature
            ->method('validate')
            ->willThrowException(new InvalidSignatureException(__('Invalid signature')));

        $this->expectExceptionMessage('Invalid signature');

        // then
        $this->subject->createFromRequest();
    }
}
