<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Model;

use Evidente\Aircash\Api\AircashClientInterface;
use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Transaction\CancellationRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Factory\HttpClientFactoryInterface;
use Evidente\Aircash\Api\SignatureInterface;
use Evidente\Aircash\Data\Transaction\CancellationRequest;
use Evidente\Aircash\Data\Transaction\CodeRequest;
use Evidente\Aircash\Exception\RequestException;
use Evidente\Aircash\Factory\HttpClientFactory;
use Evidente\Aircash\Model\AircashClient;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Psr\Log\LoggerInterface;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class AircashClientTest extends TestCase
{
    /**
     * @var Client
     */
    private $client;

    /**
     * @var MockHandler
     */
    private $mockHandler;

    /**
     * @var HttpClientFactoryInterface
     */
    private $clientFactory;

    /**
     * @var SignatureInterface|MockObject
     */
    private $signature;

    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     * @var LoggerInterface|MockObject
     */
    private $logger;

    /**
     * @var AircashClientInterface
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->clientFactory = $this->createMock(HttpClientFactory::class);

        $this->signature = $this->createMock(SignatureInterface::class);
        $this->config = $this->createMock(ConfigInterface::class);
        $this->logger = $this->createMock(LoggerInterface::class);

        $this->mockHandler = new MockHandler();

        $handlerStack = HandlerStack::create($this->mockHandler);

        $this->client = new Client(['handler' => $handlerStack]);

        $this->clientFactory->method('create')->willReturn($this->client);

        $this->subject = new AircashClient($this->clientFactory, $this->signature, $this->config, $this->logger);
    }

    /**
     * @throws RequestException
     */
    public function testGetTransactionCode()
    {
        $codeLink = 'https://www.example.com';

        // given
        $response = [
          'codeLink' => $codeLink,
        ];

        $this->mockHandler->append(
            new Response(200, [], \json_encode($response))
        );

        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'description';
        $expectedLocationId = 'location_id';
        $expectedWebsiteId = 1;

        $request = new CodeRequest(
            $expectedPartnerId,
            $expectedAmount,
            $expectedCurrency,
            $expectedPartnerTransactionId,
            $expectedDescription,
            $expectedLocationId,
            $expectedWebsiteId
        );

        $this
            ->signature
            ->expects($this->once())
            ->method('sign')
            ->with($request->toArray());

        // when
        $result = $this->subject->getTransactionCode($request);

        // then
        $this->assertSame($codeLink, $result->getCodeLink());
    }

    /**
     * @throws RequestException
     * @dataProvider transactionCodeBadResponseProvider
     */
    public function testGetTransactionCodeBadResponses(
        CodeRequestInterface $codeRequest,
        Response $response,
        string $exceptionMessage
    ) {
        $this->mockHandler->reset();

        $this->mockHandler->append($response);

        $this->expectExceptionMessage($exceptionMessage);

        $this->assertNull($this->subject->getTransactionCode($codeRequest));
    }

    /**
     * @throws RequestException
     */
    public function testCancelTransaction()
    {
        $cancelTransactionId = 'cancel-transaction-id';

        // given
        $response = [
            'CancelTransactionID' => $cancelTransactionId,
        ];

        $this->mockHandler->append(
            new Response(200, [], \json_encode($response))
        );

        $expectedPartnerId = 'partner_id';
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedWebsiteId = 1;

        $request = new CancellationRequest(
            $expectedPartnerId,
            $expectedPartnerTransactionId,
            $expectedWebsiteId
        );

        $this
            ->signature
            ->expects($this->once())
            ->method('sign')
            ->with($request->toArray());

        // when
        $result = $this->subject->cancelTransaction($request);

        // then
        $this->assertSame($cancelTransactionId, $result->getCancelTransactionID());
    }

    /**
     * @throws RequestException
     * @dataProvider cancellationBadResponseProvider
     */
    public function testCancellationBadResponses(
        CancellationRequestInterface $cancellationRequest,
        Response $response,
        string $exceptionMessage
    ) {
        $this->mockHandler->reset();

        $this->mockHandler->append($response);

        $this->expectExceptionMessage($exceptionMessage);

        $this->assertNull($this->subject->cancelTransaction($cancellationRequest));
    }

    public function transactionCodeBadResponseProvider()
    {
        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'description';
        $expectedLocationId = 'location_id';
        $expectedWebsiteId = 1;

        return [
            [
                new CodeRequest(
                    $expectedPartnerId,
                    $expectedAmount,
                    $expectedCurrency,
                    $expectedPartnerTransactionId,
                    $expectedDescription,
                    $expectedLocationId,
                    $expectedWebsiteId
                ),
                new Response(400, [], \json_encode([])),
                'Request exception'
            ],
            [
                new CodeRequest(
                    $expectedPartnerId,
                    $expectedAmount,
                    $expectedCurrency,
                    $expectedPartnerTransactionId,
                    $expectedDescription,
                    $expectedLocationId,
                    $expectedWebsiteId
                ),
                new Response(200, [], \json_encode([])),
                'Invalid code link'
            ]
        ];
    }

    public function cancellationBadResponseProvider()
    {
        $expectedPartnerId = 'partner_id';
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedWebsiteId = 1;

        return [
            [
                new CancellationRequest(
                    $expectedPartnerId,
                    $expectedPartnerTransactionId,
                    $expectedWebsiteId
                ),
                new Response(400, [], \json_encode([])),
                'Request exception'
            ],
            [
                new CancellationRequest(
                    $expectedPartnerId,
                    $expectedPartnerTransactionId,
                    $expectedWebsiteId
                ),
                new Response(200, [], \json_encode([])),
                'Invalid transaction ID'
            ]
        ];
    }
}
