<?php
// phpcs:ignorefile

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Model\Config;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Partner\ActivationRequestInterface;
use Evidente\Aircash\Model\Config\PartnerConfig;
use Magento\Config\Model\ResourceModel\Config\Data\Collection;
use Magento\Config\Model\ResourceModel\Config\Data\CollectionFactory;
use Magento\Framework\DataObject;
use Magento\Store\Model\ScopeInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class PartnerConfigTest extends TestCase
{
    /**
     * @var CollectionFactory|MockObject
     */
    private $collectionFactory;

    /**
     * @var Collection|MockObject
     */
    private $collection;

    /**
     * @var PartnerConfig
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->collectionFactory = $this->createMock(CollectionFactory::class);

        $this->collection = $this->createMock(Collection::class);

        $this->collectionFactory->method('create')->willReturn($this->collection);

        $this->subject = new PartnerConfig($this->collectionFactory);
    }

    public function testGetPartnerStatus()
    {
        // given
        $expectedPartnerStatus = ActivationRequestInterface::STATUS_PENDING;
        $expectedWebsiteId = 1;
        $expectedItem = new DataObject(
            ['value' => $expectedPartnerStatus]
        );
        $expectedTestMode = false;

        $this
            ->collection
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['scope', ScopeInterface::SCOPE_WEBSITES],
                ['scope_id', $expectedWebsiteId],
                [
                    'path',
                    ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/' . ConfigInterface::CONFIG_PATH_PARTNER_STATUS
                ]
            )
            ->willReturn($this->collection);

        $this->collection->method('count')->willReturn(1);
        $this->collection->method('getFirstItem')->willReturn($expectedItem);

        // when
        $result = $this->subject->getPartnerStatus($expectedWebsiteId, $expectedTestMode);

        // then
        $this->assertSame($expectedPartnerStatus, $result);
    }

    public function testGetPartnerStatusWillResetToNotActivatedOnNull()
    {
        // given
        $expectedPartnerStatus = ActivationRequestInterface::STATUS_NOT_ACTIVATED;
        $expectedWebsiteId = 1;
        $expectedItem = new DataObject(
            ['value' => null]
        );
        $expectedTestMode = false;

        $this
            ->collection
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['scope', ScopeInterface::SCOPE_WEBSITES],
                ['scope_id', $expectedWebsiteId],
                [
                    'path',
                    ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/' . ConfigInterface::CONFIG_PATH_PARTNER_STATUS
                ]
            )
            ->willReturn($this->collection);

        $this->collection->method('count')->willReturn(1);
        $this->collection->method('getFirstItem')->willReturn($expectedItem);

        // when
        $result = $this->subject->getPartnerStatus($expectedWebsiteId, $expectedTestMode);

        // then
        $this->assertSame($expectedPartnerStatus, $result);
    }

    public function testGetPartnerStatusWillReturnNotActivatedOnEmptyWebsiteId()
    {
        // given
        $expectedPartnerStatus = ActivationRequestInterface::STATUS_NOT_ACTIVATED;
        $expectedWebsiteId = null;
        $expectedItem = new DataObject(
            ['value' => ActivationRequestInterface::STATUS_ACTIVATED]
        );
        $expectedTestMode = false;

        $this
            ->collection
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['scope', ScopeInterface::SCOPE_WEBSITES],
                ['scope_id', $expectedWebsiteId],
                [
                    'path',
                    ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/' . ConfigInterface::CONFIG_PATH_PARTNER_STATUS
                ]
            )
            ->willReturn($this->collection);

        $this->collection->method('count')->willReturn(1);
        $this->collection->method('getFirstItem')->willReturn($expectedItem);

        // when
        $result = $this->subject->getPartnerStatus($expectedWebsiteId, $expectedTestMode);

        // then
        $this->assertSame($expectedPartnerStatus, $result);
    }

    // --------------------
    public function testGetPartnerId()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedWebsiteId = 1;
        $expectedItem = new DataObject(
            ['value' => $expectedPartnerId]
        );
        $expectedTestMode = false;

        $this
            ->collection
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['scope', ScopeInterface::SCOPE_WEBSITES],
                ['scope_id', $expectedWebsiteId],
                [
                    'path',
                    ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/' . ConfigInterface::CONFIG_PATH_PARTNER_ID
                ]
            )
            ->willReturn($this->collection);

        $this->collection->method('count')->willReturn(1);
        $this->collection->method('getFirstItem')->willReturn($expectedItem);

        // when
        $result = $this->subject->getPartnerId($expectedWebsiteId, $expectedTestMode);

        // then
        $this->assertSame($expectedPartnerId, $result);
    }

    public function testGetPartnerIdWillReturnNullOnEmptyWebsiteId()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedWebsiteId = null;
        $expectedItem = new DataObject(
            ['value' => $expectedPartnerId]
        );
        $expectedTestMode = false;

        $this
            ->collection
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['scope', ScopeInterface::SCOPE_WEBSITES],
                ['scope_id', $expectedWebsiteId],
                [
                    'path',
                    ConfigInterface::CONFIG_PATH_BASE . ConfigInterface::CONFIG_GROUP_PROD . '/' . ConfigInterface::CONFIG_PATH_PARTNER_ID
                ]
            )
            ->willReturn($this->collection);

        $this->collection->method('count')->willReturn(1);
        $this->collection->method('getFirstItem')->willReturn($expectedItem);

        // when
        $result = $this->subject->getPartnerId($expectedWebsiteId, $expectedTestMode);

        // then
        $this->assertNull($result);
    }
}
