<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Model\Payment;

use Evidente\Aircash\Api\ConfigInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\ConfirmationResultInterface;
use Evidente\Aircash\Api\TransactionProcessorInterface;
use Evidente\Aircash\Data\Transaction\ConfirmationResult;
use Evidente\Aircash\Exception\TransactionProcessorException;
use Evidente\Aircash\Model\Payment\TransactionProcessor;
use Magento\Framework\DB\Transaction;
use Magento\Framework\DB\TransactionFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory as OrderCollectionFactory;
use Magento\Sales\Model\ResourceModel\Order\Collection as OrderCollection;
use Magento\Sales\Model\Service\InvoiceService;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Psr\Log\LoggerInterface;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class TransactionProcessorTest extends TestCase
{
    /**
     * @var ConfigInterface|MockObject
     */
    private $config;

    /**
     * @var OrderCollectionFactory|MockObject
     */
    private $orderCollectionFactory;

    /**
     * @var OrderCollection|MockObject
     */
    private $orderCollection;

    /**
     * @var Transaction|MockObject
     */
    private $transaction;

    /**
     * @var TransactionFactory|MockObject
     */
    private $transactionFactory;

    /**
     * @var OrderRepositoryInterface|MockObject
     */
    private $orderRepository;

    /**
     * @var InvoiceService|MockObject
     */
    private $invoiceService;

    /**
     * @var LoggerInterface|MockObject
     */
    private $logger;

    /**
     * @var Order|MockObject
     */
    private $order;

    /**
     * @var Order\Payment|MockObject
     */
    private $payment;

    /**
     * @var Order\Invoice|MockObject
     */
    private $invoice;

    /**
     * @var TransactionProcessorInterface
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->config = $this->createMock(ConfigInterface::class);

        $this->orderCollectionFactory = $this->createMock(OrderCollectionFactory::class);
        $this->orderCollection = $this->createMock(OrderCollection::class);

        $this->orderCollectionFactory->method('create')->willReturn($this->orderCollection);

        $this->orderRepository = $this->createMock(OrderRepositoryInterface::class);
        $this->invoiceService = $this->createMock(InvoiceService::class);
        $this->logger = $this->createMock(LoggerInterface::class);

        $this->order = $this->createMock(Order::class);
        $this->payment = $this->createMock(Order\Payment::class);
        $this->invoice = $this
            ->getMockBuilder(Order\Invoice::class)
            ->disableOriginalConstructor()
            ->disableOriginalClone()
            ->disableArgumentCloning()
            ->disallowMockingUnknownTypes()
            ->setMethods(['setRequestedCaptureCase', 'setTransactionId', 'register'])
            ->getMock();

        $this->order->method('getPayment')->willReturn($this->payment);

        $this->transactionFactory = $this->createMock(TransactionFactory::class);
        $this->transaction = $this->createMock(Transaction::class);

        $this->transactionFactory->method('create')->willReturn($this->transaction);

        $this->subject = new TransactionProcessor(
            $this->config,
            $this->orderCollectionFactory,
            $this->transactionFactory,
            $this->orderRepository,
            $this->invoiceService,
            $this->logger
        );
    }

    /**
     * @throws TransactionProcessorException
     */
    public function testProcess()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->orderCollection->method('getFirstItem')->willReturn($this->order);

        $this->orderCollection->method('count')->willReturn(1);

        $this->orderRepository->method('get')->willReturn($this->order);

        $this->order->method('getGrandTotal')->willReturn($expectedAmount);

        $this->payment->expects($this->once())->method('setCcTransId')->with($expectedAircashTransactionId);

        $this->invoiceService->method('prepareInvoice')->willReturn($this->invoice);

        $this
            ->invoiceService
            ->expects($this->once())
            ->method('prepareInvoice')
            ->with($this->order);

        $this
            ->invoice
            ->expects($this->once())
            ->method('setTransactionId')
            ->with($expectedAircashTransactionId);

        $this
            ->invoice
            ->expects($this->once())
            ->method('setRequestedCaptureCase')
            ->with(Invoice::CAPTURE_ONLINE);

        $this
            ->invoice
            ->expects($this->once())
            ->method('register');

        $this
            ->transaction
            ->expects($this->at(0))
            ->method('addObject')
            ->with($this->invoice)
            ->willReturn($this->transaction);

        $this
            ->transaction
            ->expects($this->at(1))
            ->method('addObject')
            ->with($this->order)
            ->willReturn($this->transaction);

        $this->transaction->expects($this->once())->method('save');

        // then
        $this->subject->process($confirmationResult);
    }

    /**
     * @throws TransactionProcessorException
     */
    public function testProcessWithPrefix()
    {
        $this->config->method('getTransactionIdPrefix')->willReturn('prefix');

        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'prefixtransaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->orderCollection->method('getFirstItem')->willReturn($this->order);

        $this->orderCollection->method('count')->willReturn(1);

        $this->orderRepository->method('get')->willReturn($this->order);

        $this->order->method('getGrandTotal')->willReturn($expectedAmount);

        $this->payment->expects($this->once())->method('setCcTransId')->with($expectedAircashTransactionId);

        $this->invoiceService->method('prepareInvoice')->willReturn($this->invoice);

        $this
            ->invoiceService
            ->expects($this->once())
            ->method('prepareInvoice')
            ->with($this->order);

        $this
            ->invoice
            ->expects($this->once())
            ->method('setTransactionId')
            ->with($expectedAircashTransactionId);

        $this
            ->invoice
            ->expects($this->once())
            ->method('setRequestedCaptureCase')
            ->with(Invoice::CAPTURE_ONLINE);

        $this
            ->invoice
            ->expects($this->once())
            ->method('register');

        $this
            ->transaction
            ->expects($this->at(0))
            ->method('addObject')
            ->with($this->invoice)
            ->willReturn($this->transaction);

        $this
            ->transaction
            ->expects($this->at(1))
            ->method('addObject')
            ->with($this->order)
            ->willReturn($this->transaction);

        $this->transaction->expects($this->once())->method('save');

        // then
        $this->subject->process($confirmationResult);
    }

    /**
     * @throws TransactionProcessorException
     */
    public function testProcessShouldFailIfHasTransactionId()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->payment->method('getCcTransId')->willReturn($expectedAircashTransactionId);

        $this->orderCollection->method('getFirstItem')->willReturn($this->order);

        $this->orderCollection->method('count')->willReturn(1);

        $this->orderRepository->method('get')->willReturn($this->order);

        $this
            ->invoiceService
            ->expects($this->never())
            ->method('prepareInvoice')
            ->with($this->order);

        $this
            ->invoice
            ->expects($this->never())
            ->method('setTransactionId')
            ->with($expectedAircashTransactionId);

        $this
            ->invoice
            ->expects($this->never())
            ->method('setRequestedCaptureCase')
            ->with(Invoice::CAPTURE_ONLINE);

        $this
            ->invoice
            ->expects($this->never())
            ->method('register');

        $this->expectExceptionMessage('Order not found or already processed');

        // then
        $this->subject->process($confirmationResult);
    }

    /**
     * @throws TransactionProcessorException
     */
    public function testProcessShouldFailOnAmountMismatch()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->orderCollection->method('getFirstItem')->willReturn($this->order);

        $this->orderCollection->method('count')->willReturn(1);

        $this->orderRepository->method('get')->willReturn($this->order);

        $this->order->method('getGrandTotal')->willReturn(2.00);

        $this
            ->invoiceService
            ->expects($this->never())
            ->method('prepareInvoice')
            ->with($this->order);

        $this
            ->invoice
            ->expects($this->never())
            ->method('setTransactionId')
            ->with($expectedAircashTransactionId);

        $this
            ->invoice
            ->expects($this->never())
            ->method('setRequestedCaptureCase')
            ->with(Invoice::CAPTURE_ONLINE);

        $this
            ->invoice
            ->expects($this->never())
            ->method('register');

        $this->expectExceptionMessage('Order grand total amount mismatch');

        // then
        $this->subject->process($confirmationResult);
    }

    /**
     * @throws TransactionProcessorException
     */
    public function testProcessShouldHandleInvoiceException()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->orderCollection->method('getFirstItem')->willReturn($this->order);

        $this->orderCollection->method('count')->willReturn(1);

        $this->orderRepository->method('get')->willReturn($this->order);

        $this->order->method('getGrandTotal')->willReturn($expectedAmount);

        $this->payment->expects($this->once())->method('setCcTransId')->with($expectedAircashTransactionId);

        $this->invoiceService->method('prepareInvoice')->willReturn($this->invoice);

        $this
            ->invoiceService
            ->expects($this->once())
            ->method('prepareInvoice')
            ->with($this->order);

        $this
            ->invoice
            ->expects($this->once())
            ->method('setTransactionId')
            ->with($expectedAircashTransactionId);

        $this
            ->invoice
            ->expects($this->once())
            ->method('setRequestedCaptureCase')
            ->with(Invoice::CAPTURE_ONLINE);

        $this
            ->invoice
            ->expects($this->once())
            ->method('register');

        $this->invoice->method('register')->willThrowException(
            new LocalizedException(__('exception'))
        );

        $this->logger->expects($this->once())->method('alert');

        $this->expectExceptionMessage('Failed to create invoice');

        // then
        $this->subject->process($confirmationResult);
    }
}
