<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Model;

use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\ConfirmationResultInterface;
use Evidente\Aircash\Api\SignatureInterface;
use Evidente\Aircash\Exception\InvalidSignatureException;
use Evidente\Aircash\Factory\SignatureStringFactory;
use Evidente\Aircash\Model\Signature;
use PHPUnit\Framework\TestCase;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class SignatureTest extends TestCase
{
    /**
     * @var SignatureInterface
     */
    private $subject;

    private $merchantPublicKey;
    private $aircashPrivateKey;
    private $merchantPrivateKey;
    private $aircashPublicKey;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->setupKeys();

        $this->subject = new Signature(
            new SignatureStringFactory()
        );
    }

    /**
     * @throws InvalidSignatureException
     */
    public function testSign()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'Order-transaction_id';
        $expectedLocationId = 'location_id';

        $expectedInput = [
            'PartnerID' => $expectedPartnerId,
            'Amount' => $expectedAmount,
            'CurrencyID' => $expectedCurrency,
            'PartnerTransactionID' => $expectedPartnerTransactionId,
            'Description' => $expectedDescription,
            'LocationID' => $expectedLocationId,
        ];

        // then
        $signature = $this->subject->sign($expectedInput, $this->merchantPrivateKey);

        $expectedInput['Signature'] = $signature;

        $this->assertNull($this->subject->validate($expectedInput, $signature, $this->merchantPublicKey));
    }

    /**
     * @throws InvalidSignatureException
     */
    public function testValidate()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
        ];

        // then
        $signature = $this->subject->sign($expectedInput, $this->aircashPrivateKey);

        $expectedInput[ConfirmationResultInterface::FIELD_SIGNATURE] = $signature;

        $this->assertNull($this->subject->validate($expectedInput, $signature, $this->aircashPublicKey));
    }

    /**
     * @throws InvalidSignatureException
     */
    public function testValidateShouldThrowOnFailure()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
        ];

        // then
        $this->subject->sign($expectedInput, $this->aircashPrivateKey);

        $signature = 'bad signature';

        $expectedInput[ConfirmationResultInterface::FIELD_SIGNATURE] = $signature;

        $this->expectExceptionMessage('Invalid signature');

        $this->subject->validate($expectedInput, $signature, $this->aircashPublicKey);
    }

    private function setupKeys()
    {
        $merchantData = [
            'countryName' => 'HR',
            'localityName' => 'Zagreb',
            'organizationName' => 'Organization',
            'organizationalUnitName' => 'Organization Unit Name',
            'commonName' => 'Common Name',
            'emailAddress' => 'org@example.com'
        ];

        $merchantPrivateKey = openssl_pkey_new(
            [
                'private_key_bits' => 4096,
                'private_key_type' => OPENSSL_KEYTYPE_RSA,
            ]
        );

        $merchantCsr = openssl_csr_new($merchantData, $merchantPrivateKey, ['digest_alg' => 'sha256']);

        $merchantX509 = openssl_csr_sign(
            $merchantCsr,
            null,
            $merchantPrivateKey,
            7,
            ['digest_alg' => 'sha256']
        );

        openssl_x509_export($merchantX509, $merchantPublicKeyExport);
        $this->merchantPublicKey = $merchantPublicKeyExport;

        openssl_pkey_export($merchantPrivateKey, $merchantPrivateKeyExport);
        $this->merchantPrivateKey = $merchantPrivateKeyExport;

        $aircashData = [
            'countryName' => 'HR',
            'localityName' => 'Zagreb',
            'organizationName' => 'Aircash',
            'organizationalUnitName' => 'Aircash Unit Name',
            'commonName' => 'Common Name',
            'emailAddress' => 'aircash@example.com'
        ];

        $aircashPrivateKey = openssl_pkey_new(
            [
                'private_key_bits' => 4096,
                'private_key_type' => OPENSSL_KEYTYPE_RSA,
            ]
        );

        $aircashCsr = openssl_csr_new($aircashData, $aircashPrivateKey, ['digest_alg' => 'sha256']);

        $aircashX509 = openssl_csr_sign(
            $aircashCsr,
            null,
            $aircashPrivateKey,
            7,
            ['digest_alg' => 'sha256']
        );

        openssl_x509_export($aircashX509, $aircashPublicKey);
        $this->aircashPublicKey = $aircashPublicKey;

        openssl_pkey_export($aircashPrivateKey, $aircashPrivateKeyExport);
        $this->aircashPrivateKey = $aircashPrivateKeyExport;
    }
}
