<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Rest;

use Evidente\Aircash\Api\AircashClientInterface;
use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Factory\Transaction\CodeRequestFactoryInterface;
use Evidente\Aircash\Api\Rest\CheckoutDataProviderInterface;
use Evidente\Aircash\Data\Transaction\CodeRequest;
use Evidente\Aircash\Data\Transaction\CodeResult;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Exception\RequestException;
use Evidente\Aircash\Rest\CheckoutSessionDataProvider;
use Magento\Checkout\Model\Session;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Psr\Log\LoggerInterface;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class CheckoutSessionDataProviderTest extends TestCase
{
    /**
     * @var OrderRepositoryInterface|MockObject
     */
    private $orderRepository;

    /**
     * @var Session|MockObject
     */
    private $session;

    /**
     * @var CodeRequestFactoryInterface|MockObject
     */
    private $codeRequestFactory;

    /**
     * @var LoggerInterface|MockObject
     */
    private $logger;

    /**
     * @var AircashClientInterface|MockObject
     */
    private $aircashClient;

    /**
     * @var Order|MockObject
     */
    private $order;

    /**
     * @var Order\Payment|MockObject
     */
    private $payment;

    /**
     * @var CheckoutDataProviderInterface
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->orderRepository = $this->createMock(OrderRepositoryInterface::class);
        $this->session = $this
            ->getMockBuilder(Session::class)
            ->disableOriginalConstructor()
            ->disableOriginalClone()
            ->disableArgumentCloning()
            ->disallowMockingUnknownTypes()
            ->setMethods(['getLastRealOrderId', 'getLastOrderId'])
            ->getMock();

        $this->codeRequestFactory = $this->createMock(CodeRequestFactoryInterface::class);
        $this->logger = $this->createMock(LoggerInterface::class);
        $this->aircashClient = $this->createMock(AircashClientInterface::class);

        $this->order = $this->createMock(Order::class);
        $this->payment = $this->createMock(Order\Payment::class);

        $this->order->method('getPayment')->willReturn($this->payment);

        $this->subject = new CheckoutSessionDataProvider(
            $this->orderRepository,
            $this->session,
            $this->codeRequestFactory,
            $this->aircashClient,
            $this->logger
        );
    }

    /**
     * @throws LocalizedException
     */
    public function testGetCode()
    {
        // given
        $orderId = '100000';

        $this->session->method('getLastOrderId')->willReturn($orderId);

        $this->orderRepository->method('get')->willReturn($this->order);

        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'description';
        $expectedLocationId = 'location_id';
        $expectedWebsiteId = 1;

        $codeRequest = new CodeRequest(
            $expectedPartnerId,
            $expectedAmount,
            $expectedCurrency,
            $expectedPartnerTransactionId,
            $expectedDescription,
            $expectedLocationId,
            $expectedWebsiteId
        );

        $this
            ->codeRequestFactory
            ->method('createFromOrder')
            ->willReturn($codeRequest);

        $expectedLink = 'https://www.example.com';

        $codeResult = new CodeResult($expectedLink);

        $this->aircashClient->method('getTransactionCode')->willReturn($codeResult);

        // when
        $result = $this->subject->getCode();

        // then
        $this->assertSame($expectedLink, $result->getCodeLink());
    }

    /**
     * @throws LocalizedException
     */
    public function testGetCodeShouldFailBecauseLastRealOrderIdIsMissing()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'description';
        $expectedLocationId = 'location_id';
        $expectedWebsiteId = 1;

        $codeRequest = new CodeRequest(
            $expectedPartnerId,
            $expectedAmount,
            $expectedCurrency,
            $expectedPartnerTransactionId,
            $expectedDescription,
            $expectedLocationId,
            $expectedWebsiteId
        );

        $this
            ->codeRequestFactory
            ->method('createFromOrder')
            ->willReturn($codeRequest);

        $this->expectExceptionMessage('Order not found');

        // then
        $this->subject->getCode();
    }

    /**
     * @throws LocalizedException
     */
    public function testGetCodeShouldFailBecauseOrderNotFound()
    {
        // given
        $orderId = '100000';

        $this->session->method('getLastOrderId')->willReturn($orderId);

        $this->orderRepository->expects($this->once())->method('get')->willThrowException(
            new NoSuchEntityException()
        );

        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'description';
        $expectedLocationId = 'location_id';
        $expectedWebsiteId = 1;

        $codeRequest = new CodeRequest(
            $expectedPartnerId,
            $expectedAmount,
            $expectedCurrency,
            $expectedPartnerTransactionId,
            $expectedDescription,
            $expectedLocationId,
            $expectedWebsiteId
        );

        $this
            ->codeRequestFactory
            ->method('createFromOrder')
            ->willReturn($codeRequest);

        $this->expectExceptionMessage('Order not found');

        // then
        $this->subject->getCode();
    }

    /**
     * @throws LocalizedException
     */
    public function testGetCodeShouldFailBecauseOfCodeRequestFactoryException()
    {
        // given
        $orderId = '100000';

        $this->session->method('getLastOrderId')->willReturn($orderId);

        $this->orderRepository->method('get')->willReturn($this->order);

        $this
            ->codeRequestFactory
            ->expects($this->once())
            ->method('createFromOrder')
            ->willThrowException(
                new InvalidInputException(__('Invalid input'))
            );

        $this
            ->logger
            ->expects($this->once())
            ->method('alert')
            ->with(
                'Aircash checkout session data provider exception on order #: Invalid input'
            );

        $this->expectExceptionMessage('Invalid input');

        // then
        $this->subject->getCode();
    }

    /**
     * @throws LocalizedException
     */
    public function testGetCodeShouldFailOnAircashClientException()
    {
        // given
        $orderId = '100000';

        $this->session->method('getLastOrderId')->willReturn($orderId);

        $this->orderRepository->method('get')->willReturn($this->order);

        $expectedPartnerId = 'partner_id';
        $expectedAmount = 5.00;
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedDescription = 'description';
        $expectedLocationId = 'location_id';
        $expectedWebsiteId = 1;

        $codeRequest = new CodeRequest(
            $expectedPartnerId,
            $expectedAmount,
            $expectedCurrency,
            $expectedPartnerTransactionId,
            $expectedDescription,
            $expectedLocationId,
            $expectedWebsiteId
        );

        $this
            ->codeRequestFactory
            ->method('createFromOrder')
            ->willReturn($codeRequest);

        $this
            ->aircashClient
            ->method('getTransactionCode')
            ->willThrowException(
                new RequestException(__('Request exception'))
            );

        $this->expectExceptionMessage('Request exception');

        // then
        $this->subject->getCode();
    }
}
