<?php

declare(strict_types=1);

namespace Evidente\Aircash\Test\Unit\Rest;

use Evidente\Aircash\Api\Data\Transaction\CodeRequestInterface;
use Evidente\Aircash\Api\Data\Transaction\ConfirmationResultInterface;
use Evidente\Aircash\Api\Factory\Transaction\ConfirmationResultFactoryInterface;
use Evidente\Aircash\Api\Rest\TransactionConfirmationHandlerInterface;
use Evidente\Aircash\Api\TransactionProcessorInterface;
use Evidente\Aircash\Data\Transaction\ConfirmationResult;
use Evidente\Aircash\Exception\InvalidInputException;
use Evidente\Aircash\Exception\InvalidSignatureException;
use Evidente\Aircash\Exception\TransactionProcessorException;
use Evidente\Aircash\Rest\TransactionConfirmationHandler;
use Magento\Framework\Webapi\Exception;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use ReflectionException;

/**
 * @SuppressWarnings(PHPMD)
 */
class TransactionConfirmationHandlerTest extends TestCase
{
    /**
     * @var ConfirmationResultFactoryInterface|MockObject
     */
    private $confirmationResultFactory;

    /**
     * @var TransactionProcessorInterface|MockObject
     */
    private $transactionProcessor;

    /**
     * @var TransactionConfirmationHandlerInterface
     */
    private $subject;

    /**
     * @throws ReflectionException
     */
    protected function setUp(): void
    {
        $this->confirmationResultFactory = $this->createMock(ConfirmationResultFactoryInterface::class);
        $this->transactionProcessor = $this->createMock(TransactionProcessorInterface::class);

        $this->subject = new TransactionConfirmationHandler(
            $this->confirmationResultFactory,
            $this->transactionProcessor
        );
    }

    /**
     * @throws Exception
     */
    public function testExecute()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->confirmationResultFactory->method('createFromRequest')->willReturn($confirmationResult);

        // when
        $response = $this->subject->execute();

        // then
        $this->assertSame('OK', $response->getMessage());
    }

    /**
     * @throws Exception
     */
    public function testExecuteShouldFailBecauseOfInvalidInput()
    {
        // given
        $this
            ->confirmationResultFactory
            ->expects($this->once())
            ->method('createFromRequest')
            ->willThrowException(
                new InvalidInputException(__('Invalid input'))
            );

        $this->expectExceptionMessage('Invalid input');

        // then
        $this->subject->execute();
    }

    /**
     * @throws Exception
     */
    public function testExecuteShouldFailBecauseOfInvalidSignature()
    {
        // given
        $this
            ->confirmationResultFactory
            ->expects($this->once())
            ->method('createFromRequest')
            ->willThrowException(
                new InvalidSignatureException(__('Invalid signature'))
            );

        $this->expectExceptionMessage('Invalid signature');

        // then
        $this->subject->execute();
    }

    /**
     * @throws Exception
     */
    public function testExecuteShouldFailOnProcessorException()
    {
        // given
        $expectedPartnerId = 'partner_id';
        $expectedCurrency = CodeRequestInterface::CURRENCY_MAPPING[CodeRequestInterface::CURRENCY_EUR];
        $expectedAmount = 5.00;
        $expectedPartnerTransactionId = 'transaction_id';
        $expectedAircashTransactionId = 'aircash_transaction_id';
        $expectedSignature = 'signature';

        $expectedInput = [
            ConfirmationResultInterface::FIELD_PARTNER_TRANSACTION_ID => $expectedPartnerTransactionId,
            ConfirmationResultInterface::FIELD_CURRENCY_ID => $expectedCurrency,
            ConfirmationResultInterface::FIELD_AMOUNT => $expectedAmount,
            ConfirmationResultInterface::FIELD_PARTNER_ID => $expectedPartnerId,
            ConfirmationResultInterface::FIELD_AIRCASH_TRANSACTION_ID => $expectedAircashTransactionId,
            ConfirmationResultInterface::FIELD_SIGNATURE => $expectedSignature,
        ];

        $confirmationResult = new ConfirmationResult($expectedInput);

        $this->confirmationResultFactory->method('createFromRequest')->willReturn($confirmationResult);

        $this
            ->transactionProcessor
            ->expects($this->once())
            ->method('process')
            ->willThrowException(
                new TransactionProcessorException(__('Transaction exception'))
            );

        $this->expectExceptionMessage('Transaction exception');

        // then
        $this->subject->execute();
    }
}
